import pydas.exceptions
import time

def reauth(fn):
    """this decorator will detect a stale token and renew the token if possible,
    then retry the failed api call.
    """
    from functools import wraps
    @wraps(fn)
    def wrapper(*args, **kw):
        try:
            retVal = fn(*args, **kw)
            return retVal
        except pydas.exceptions.PydasException as detail:
            print "Caught PydasException: ", detail
            print "Waiting 30 seconds, then retrying request"

            # wait 30 seconds before retrying
            time.sleep(30)

            # renew the token
            # get the instance of the CoreDriver and set it as "that"
            that = args[0]
            token = that.login_with_api_key(that.__class__.email,
                                            that.__class__.apikey)

            # now fix up the arguments of the original call to use the renewed token
            argsList = list(args)
            argsList[2]['token'] = token
            args = tuple(argsList)

            # try the api call again
            retVal = fn(*args, **kw)
            return retVal
    return wrapper



"""Modules for defining exceptions in pydas
"""

class PydasException(Exception):
    """Base class for exception to throw within pydas
    """

    def __init__(self, value):
        """Override the constructor to support a basic message
        """
        super(PydasException, self).__init__()
        self.value = value

    def __str__(self):
        """Override the string method
        """
        return repr(self.value)



from distutils.core import setup
import pydas
setup(name='pydas',
      version=pydas.version,
      packages=['pydas'],
      author='Patrick Reynolds',
      author_email='patrick.reynolds@kitware.com',
      url='http://github.com/cpatrick/pydas',
      install_requires=['requests','simplejson']
      )
